
#ifndef VTKRENDERINGFREETYPE_EXPORT_H
#define VTKRENDERINGFREETYPE_EXPORT_H

#ifdef VTKRENDERINGFREETYPE_STATIC_DEFINE
#  define VTKRENDERINGFREETYPE_EXPORT
#  define VTKRENDERINGFREETYPE_NO_EXPORT
#else
#  ifndef VTKRENDERINGFREETYPE_EXPORT
#    ifdef RenderingFreeType_EXPORTS
        /* We are building this library */
#      define VTKRENDERINGFREETYPE_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define VTKRENDERINGFREETYPE_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef VTKRENDERINGFREETYPE_NO_EXPORT
#    define VTKRENDERINGFREETYPE_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef VTKRENDERINGFREETYPE_DEPRECATED
#  define VTKRENDERINGFREETYPE_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef VTKRENDERINGFREETYPE_DEPRECATED_EXPORT
#  define VTKRENDERINGFREETYPE_DEPRECATED_EXPORT VTKRENDERINGFREETYPE_EXPORT VTKRENDERINGFREETYPE_DEPRECATED
#endif

#ifndef VTKRENDERINGFREETYPE_DEPRECATED_NO_EXPORT
#  define VTKRENDERINGFREETYPE_DEPRECATED_NO_EXPORT VTKRENDERINGFREETYPE_NO_EXPORT VTKRENDERINGFREETYPE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKRENDERINGFREETYPE_NO_DEPRECATED
#    define VTKRENDERINGFREETYPE_NO_DEPRECATED
#  endif
#endif
/* AutoInit dependencies. */
#include "vtkRenderingCoreModule.h"


/* AutoInit implementations. */
#ifdef vtkRenderingFreeType_AUTOINIT_INCLUDE
#include vtkRenderingFreeType_AUTOINIT_INCLUDE
#endif
#ifdef vtkRenderingFreeType_AUTOINIT
#include "vtkAutoInit.h"
VTK_MODULE_AUTOINIT(vtkRenderingFreeType)
#endif

#endif /* VTKRENDERINGFREETYPE_EXPORT_H */
