/*=========================================================================

  Program:   Visualization Toolkit
  Module:    vtkVTK_USE_SCALED_SOA_ARRAYS.h

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

// defined if VTK provides the vtkScaledSOADataArray classes
/* #undef VTK_USE_SCALED_SOA_ARRAYS */
