/*=========================================================================

  Program:   Visualization Toolkit
  Module:    vtkChemistryConfigure.h.in

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

#ifndef vtkDomainsChemistryConfigure_h
#define vtkDomainsChemistryConfigure_h

/* Where the Blue Obelisk Data Repository element.xml file is found. */
#define VTK_BODR_DATA_PATH_BUILD "/private/var/folders/nz/j6p8yfhx1mv_0grj5xl4650h0000gp/T/abs_a2z_sn0h6g/croot/vtk_1679752185033/work/Domains/Chemistry"

#endif
